#ifndef TINSEL_PLUGIN_H
#define TINSEL_PLUGIN_H

#include "common/util.h"
#include "base/game.h"
#include "base/plugins.h"

namespace Tinsel {

struct GameSettings {
	const char *gameid;
	const char *description;
};

struct TinselGameSettings {
	const char *name;
	const char *description;
	const char *detectfile;

	GameSettings toGameSettings() const {
		const GameSettings dummy = { name, description };
		return dummy;
	}
};

struct TinselLanguage {
	Common::Language language;
	const char *detectfile;
};

//DetectedGameList Engine_TINSEL_detectGames(const FSList &fslist);

} // namespace Tinsel

#endif /* TINSEL_PLUGIN_H */
