#ifndef _TINSEL_INDEX_H
#define _TINSEL_INDEX_H

#include "common/array.h"

namespace Tinsel {

struct IndexEntry {
	char filename[12];
	uint32 i_shifted;
	uint32 flags;
};

class Index {
	Common::Array<IndexEntry> _index;

public:
	Index(void);
	virtual ~Index(void);
};

} // namespace Tinsel

#endif
