/* ScummVM - Scumm Interpreter
 * Copyright (C) 2002-2005 The ScummVM project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /home9/user/studenten/dahms/c-source/cvs_root/tinsel/bitmap.h,v 1.2 2005/05/03 09:21:22 dahms Exp $
 */

#ifndef TINSEL_BITMAP_H
#define TINSEL_BITMAP_H

#include "tinsel/types.h"

namespace Tinsel {

class Bitmap {
	uint16 _width;
	uint16 _height;
	const SCNPalette *_palette;

	uint8 *_bitmap;

public:
	Bitmap(uint16 width, uint16 height, uint16 *blocklist,
		const SCNPalette *pal, SCNPixelBlocks *blocks);
	Bitmap(uint16 width, uint16 height, uint8 *pixels,
		const SCNPalette *pal);
	virtual ~Bitmap();

	uint8 *getBitmap() { return _bitmap; }
	const SCNPalette *getPalette() { return _palette; }
	uint16 getWidth() { return _width; }
	uint16 getHeight() { return _height; }
};

} // namespace Tinsel

#endif // TINSEL_BITMAP_H
