/* ScummVM - Scumm Interpreter
 * Copyright (C) 2002-2005 The ScummVM project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /home9/user/studenten/dahms/c-source/cvs_root/tinsel/tinsel.h,v 1.8 2006/09/23 18:11:17 dahms Exp $
 */

#ifndef TINSEL_TINSEL_H
#define TINSEL_TINSEL_H

#include "common/stdafx.h"
#include "common/system.h"
#include "engines/engine.h"
#include "base/game.h"

#include "tinsel/plugin.h"
#include "tinsel/types.h"
#include "tinsel/bitmap.h"
#include "tinsel/index.h"
#include "tinsel/sound.h"
#include "tinsel/midi.h"

namespace Tinsel {

class TinselEngine : public Engine {
	uint32 _features;
	Index *_index;
	Sound *_sound;
	Midi *_midi;

	void errorString(const char *buf_input, char *buf_output);
	void handleEvents();

protected:
	int go();
	bool initGame();
	int init();

public:
	TinselEngine(OSystem *syst);
	virtual ~TinselEngine();

	int getFeatures();
	bool setBackgroundBitmap(Bitmap *bitmap);

	void shutdown();
};

} // namespace Tinsel

#endif /* TINSEL_TINSEL_H */
