/* ScummVM - Scumm Interpreter
 * Copyright (C) 2002-2005 The ScummVM project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /home9/user/studenten/dahms/c-source/cvs_root/tinsel/types.h,v 1.3 2006/09/07 10:39:56 dahms Exp $
 */

#ifndef TINSEL_TYPES_H
#define TINSEL_TYPES_H

#include "common/stdafx.h"
#include "common/scummsys.h"

namespace Tinsel {

struct SCNPixelBlocks {
	uint16 num;
	uint8 **blocks;
};

struct SCNPalette {
	uint32 offset;
	uint8 palette[1024];
};

}

#endif // TINSEL_TYPES_H
