#ifndef TINSEL_SCRIPT_H
#define TINSEL_SCRIPT_H

namespace Tinsel {

class Script {
	uint8 *_buf;
	uint32 _buflen;

	uint16 get_uint16(uint32 pos);
	uint32 get_uint32(uint32 pos);

public:
	Script(uint8 *buf, uint32 len);
	~Script(void);

	void run(void);
};

} // namespace Tinsel

#endif
