#ifndef TINSEL_SOUND_H
#define TINSEL_SOUND_H

#include "common/stdafx.h"
#include "common/file.h"
#include "sound/mixer.h"

namespace Tinsel {

class Sound {
	Common::File _idxfile;
	Common::File _smpfile;
	Audio::Mixer *_mixer;
	Audio::SoundHandle _sfxHandle;
	uint32 *_offsets;

public:
	Sound(Audio::Mixer *mixer);
	virtual ~Sound();

	bool playSound(uint32 index);
	bool playSoundOffset(uint32 offset);
};

} // namespace Tinsel

#endif // TINSEL_SOUND_H

